/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */


package org.openmobileis.common.util.collection;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Use to store an Array of int.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class IntArray  implements java.io.Serializable {
	protected static final long serialVersionUID = 5521257935120563452L;

  /**
     * The array buffer into which the elements of the ArrayList are stored.
     * The capacity of the ArrayList is the length of this array buffer.
     */
    private int elementData[];

    /**
     * The size of the ArrayList (the number of elements it contains).
     */
    private int size;

    /**
     * Constructs an empty list with the specified initial capacity.
     *
     * @param   initialCapacity   the initial capacity of the list.
     * @exception IllegalArgumentException if the specified initial capacity
     *            is negative
     */
    public IntArray(int initialCapacity) {
      super();
        if (initialCapacity < 0)
            throw new IllegalArgumentException("Illegal Capacity: "+
                                               initialCapacity);
      this.elementData = new int[initialCapacity];
    }

    /**
     * Constructs an empty list.
     */
    public IntArray() {
      this(10);
    }

    public void clear() {
      this.elementData = new int[this.elementData.length];
      size = 0;
    }

    protected int[] getArrayElements() {
      return elementData;
    }

    protected void setArrayElements(int[] array) {
      elementData = array;
    }


    /**
     * Increases the capacity of this <tt>ArrayList</tt> instance, if
     * necessary, to ensure  that it can hold at least the number of elements
     * specified by the minimum capacity argument.
     *
     * @param   minCapacity   the desired minimum capacity.
     */
    public void ensureCapacity(int minCapacity) {
  int oldCapacity = elementData.length;
  if (minCapacity > oldCapacity) {
      int oldData[] = elementData;
      int newCapacity = (oldCapacity * 3)/2 + 1;
          if (newCapacity < minCapacity)
    newCapacity = minCapacity;
      elementData = new int[newCapacity];
      System.arraycopy(oldData, 0, elementData, 0, size);
  }
    }

    /**
     * Returns the number of elements in this list.
     *
     * @return  the number of elements in this list.
     */
    public int size() {
  return size;
    }

    /**
     * Tests if this list has no elements.
     *
     * @return  <tt>true</tt> if this list has no elements;
     *          <tt>false</tt> otherwise.
     */
    public boolean isEmpty() {
  return size == 0;
    }

  	/**
  	 * return true if the array contains the specified int false otherwise.
  	 *
  	 * @param int o : the int to be tested.
  	 * @return <tt>true</tt> if the specified int is inside the array.
  	 */
  	public boolean contains(int o) {
  		for (int i=0; i<size; i++)	{
  			if (elementData[i] == o)	{
  				return true;
  			}
  		}
  		return false;
  	}


    /**
     * Returns an array containing all of the elements in this list
     * in the correct order.
     *
     * @return an array containing all of the elements in this list
     * 	       in the correct order.
     */
    public int[] toArray() {
      int[] result = new int[size];
      System.arraycopy(elementData, 0, result, 0, size);
      return result;
    }


    // Positional Access Operations

    /**
     * Returns the element at the specified position in this list.
     *
     * @param  index index of element to return.
     * @return the element at the specified position in this list.
     * @throws    IndexOutOfBoundsException if index is out of range <tt>(index
     * 		  &lt; 0 || index &gt;= size())</tt>.
     */
    public int get(int index) {
      RangeCheck(index);

      return elementData[index];
    }


    /**
     * Appends the specified element to the end of this list.
     *
     * @param o element to be appended to this list.
     * @return <tt>true</tt> (as per the general contract of Collection.add).
     */
    public boolean add(int o) {
      ensureCapacity(size + 1);  // Increments modCount!!
      elementData[size++] = o;
      return true;
    }

    /**
     * Appends the specified LongArray to the end of this list.
     *
     * @param array long to add.
     * @return <tt>true</tt> (as per the general contract of Collection.add).
     */
    public boolean add(IntArray  array) {
      ensureCapacity(size + array.size());  // Increments modCount!!
      System.arraycopy(array.getArrayElements(), 0, elementData, size,
               array.size());
      size+= array.size();
      return true;
    }

	/**
	 * replace at the specified index the element.
	 * if there is no element at index add the new one at the end of the array.
	 *
	 * @param index index in the array where element is replaced.
	 * @param o element to be replaced to this ArrayList.
	 * @return <tt>true</tt>.
	 */
	public boolean replace(int index, int o) {
		if (index >= size)	{
			return this.add(o);
		}
		elementData[index] = o;
		return true;
	}

    /**
     * Appends the specified LongArray to the end of this list.
     *
     * @param array long to add.
     * @return <tt>true</tt> (as per the general contract of Collection.add).
     */
    public boolean add(int[]  array) {
      ensureCapacity(size + array.length);  // Increments modCount!!
      System.arraycopy(array, 0, elementData, size,
               array.length);
      size+= array.length;
      return true;
    }

    /**
     * Appends the specified element to the end of this list.
     *
     * @param o element to be appended to this list.
     * @return <tt>true</tt> (as per the general contract of Collection.add).
     */
    public boolean add(int index, int o) {
      ensureCapacity(index);  // Increments modCount!!
      elementData[index] = o;
            if (index >= size)
                size = index+1;
      return true;
    }



    /**
     * Removes the element at the specified position in this list.
     * Shifts any subsequent elements to the left (subtracts one from their
     * indices).
     *
     * @param index the index of the element to removed.
     * @return the element that was removed from the list.
     * @throws    IndexOutOfBoundsException if index out of range <tt>(index
     * 		  &lt; 0 || index &gt;= size())</tt>.
     */
    public long remove(int index) {
      RangeCheck(index);
      long oldValue = elementData[index];

      int numMoved = size - index - 1;
      if (numMoved > 0)
          System.arraycopy(elementData, index+1, elementData, index,
               numMoved);
      elementData[--size] = 0; // Let gc do its work

      return oldValue;
    }




    /**
     * Check if the given index is in range.  If not, throw an appropriate
     * runtime exception.
     */
    private void RangeCheck(int index) {
      if (index >= size || index < 0)
          throw new IndexOutOfBoundsException(
        "Index: "+index+", Size: "+size);
    }
}